<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2013 Michel Baeriswyl <formhandler_backend@pragmas.ch>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *
 *
 *   47: class tx_formhandlerbackend_addFlexformElement extends t3lib_extFileFunctions
 *   61:     function __construct()
 *   79:     function checkPretemplate($dir)
 *   97:     function addFields_formselect($config)
 *
 * TOTAL FUNCTIONS: 3
 * (This index is automatically created/updated by the extension "extdeveval")
 *
 */


/**
 * Class that adds the dynamic form select to plugin flexform.
 *
 * @author	Michel Baeriswyl <formhandler_backend@pragmas.ch>
 * @package	TYPO3
 * @subpackage	tx_formhandlerbackend
 */
class tx_formhandlerbackend_addFlexformElement extends t3lib_extFileFunctions {
  // configuration values
  var $sysconf = array();
  var $pretemplates = '';
  var $htmlTemplate = '';
  var $userMailTemplate = '';
  var $adminMailTemplate = '';
  var $marker = '';

	/**
	 * Constructor, sets class configuration
	 *
	 * @return	void
	 */
  function __construct() {
    $this->sysconf = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['formhandler_backend']);
    $this->pretemplates = $this->sysconf['pretemplates'];
    $this->tscTemplate = $this->sysconf['ts-contants'];
    $this->tssTemplate = $this->sysconf['ts-config'];
    $this->htmlTemplate = $this->sysconf['html-template'];
    $this->userMailTemplate = $this->sysconf['mailuser-template'];
    $this->adminMailTemplate = $this->sysconf['mailadmin-template'];
    $this->langTemplate = $this->sysconf['lang-template'];
    $this->marker = $this->sysconf['marker'];
  }

	/**
	 * Checks whether the necessary template-files exist.
	 *
	 * @param	string		absolute path to the pretemplate directory
	 * @return	boolean		returns true, when ts-, html- and lang-templates exist
	 */
  function checkPretemplate($dir) {
    $tscPath = $dir.$this->tscTemplate;
    if (!($tscPath && t3lib_div::validPathStr($tscPath) && @is_file($tscPath))) return false;
    $tssPath = $dir.$this->tssTemplate;
    if (!($tssPath && t3lib_div::validPathStr($tssPath) && @is_file($tssPath))) return false;
    $htmlPath = $dir.$this->htmlTemplate;
    if (!($htmlPath && t3lib_div::validPathStr($htmlPath) && @is_file($htmlPath))) return false;
    $langTemplate = $dir.$this->langTemplate;
    if (!($langTemplate && t3lib_div::validPathStr($langTemplate) && @is_file($langTemplate))) return false;
    return true;
  }

	/**
	 * Sets the items for the "formselect" dropdown.
	 *
	 * @param	array		$config
	 * @return	array		The config including the items for the dropdown
	 */
	function addFields_formselect($config) {

		global $LANG;

		$pid = $config['row']['pid'];
		$optionList = array();
		$optionList[] = array(0 => $LANG->sL('LLL:EXT:formhandler_backend/pi1/locallang.xml:pi_flexform.forms_pleaseselect'), 1 => '');
		$optionList[] = array(0 => '-----------------------', 1 => '0');

    $path = t3lib_div::getFileAbsFileName($this->pretemplates);
    if (@t3lib_basicFileFunctions::is_directory($path)) {
      $directories = t3lib_div::get_dirs($path);
      if (is_array($directories))    {
        foreach($directories as $dirName) {
          if ($this->checkPretemplate($path.$dirName.'/'))
            $optionList[] = array(0 => $dirName, 1 => $this->pretemplates . $dirName);
        }
      }
    }

		$config['items'] = array_merge($config['items'],$optionList);

		return $config;
	}

}

?>