<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2013 Michel Baeriswyl <formhandler_backend@pragmas.ch>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *
 *
 *   63: class tx_formhandlerbackend_tcemainprocdm extends t3lib_extFileFunctions
 *   91:     function __construct()
 *  122:     function convertTyposcriptToArray($ts)
 *  140:     public function convertTypoScriptArrayToPlainArray(array $typoScriptArray)
 *  174:     public function convertPlainArrayToTypoScriptArray(array $plainArray)
 *  199:     function replaceArray($searchKey,$replaceValue,array $input)
 *  219:     function array_keys_multi(array $array)
 *  238:     function convertArrayToTyposcript($tsArr,$insTab=0)
 *  262:     function getNextId($table)
 *  276:     function readTemplate($path,$marker)
 *  292:     function writeFile($path,$content)
 *  304:     function readFlexform($flexform)
 *  325:     function reverseRmDir($path)
 *  358:     function reverseCopyDir($fromPath,$toPath)
 *  401:     function getLangOverlay($uid,$table,$columns)
 *  427:     function assembleField($fieldRow,$options,$step)
 *  510:     function assembleOption($fieldRow,$typeRow,$optionRow,$step)
 *  535:     function processCmdmap_preProcess ($cmd, $table, $id, $value, &$reference)
 *  565:     function processDatamap_postProcessFieldArray ($status, $table, $id, &$fieldArray, &$reference)
 *  679:     function processDatamap_afterDatabaseOperations (&$status, &$table, &$id, &$fieldArray, &$reference)
 *
 * TOTAL FUNCTIONS: 19
 * (This index is automatically created/updated by the extension "extdeveval")
 *
 */

/**
 * Class that adds functions to build formhandler templates from pretemplates
 * based on db entries for dynamic formparts (called fieldsets), fields, options, type and validation.
 *
 * @author	Michel Baeriswyl <formhandler_backend@pragmas.ch>
 * @package	TYPO3
 * @subpackage	tx_formhandlerbackend
 */
class tx_formhandlerbackend_tcemainprocdm extends \TYPO3\CMS\Core\Utility\File\ExtendedFileUtility {
  // configuration values
  var $sysConf = array();
  var $pretemplatesFolder = '';
  var $templatesFolder = '';
  var $tsConstantsUrl = '';
  var $tsSetupUrl = '';
  var $htmlTemplateUrl = '';
  var $mailUserTemplateUrl = '';
  var $mailAdminTemplateUrl = '';
  var $langTemplateUrl = '';
  var $languages = array();
  var $langLabels = array();

  var $mailContent = '';
  var $langContent = array();
  var $tsContent = array();
  var $marker = '';
  var $sysTemplateWrite = FALSE;

  var $tsResult = '';
  var $tsArray = array();

	/**
	 * Constructor, sets class configuration
	 *
	 * @return	void
	 */
  function __construct() {
      // define constants
    $this->sysConf = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['formhandler_backend']);
    $this->pretemplatesFolder = $this->sysConf['pretemplates'];
    $this->templatesFolder = $this->sysConf['templates'];
    $this->tsConstantsUrl = $this->sysConf['ts-contants'];
    $this->tsSetupUrl = $this->sysConf['ts-config'];
    $this->htmlTemplateUrl = $this->sysConf['html-template'];
    $this->mailUserTemplateUrl = $this->sysConf['mailuser-template'];
    $this->mailAdminTemplateUrl = $this->sysConf['mailadmin-template'];
    $this->langTemplateUrl = $this->sysConf['lang-template'];
    $this->marker = str_replace('###','',$this->sysConf['marker']);
    $this->sysTemplateWrite = $this->sysConf['sys_template_write'];

      // get sys_languages abbreviation from flag
    $this->languages[0] = 'default';
    $langQuery = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid,flag','sys_language','hidden=0');
    while (false!==($langRow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($langQuery))) {
      $sysLangId = $langRow['uid'];
      $this->languages[$sysLangId] = $langRow['flag'];
    }
    $GLOBALS['TYPO3_DB']->sql_free_result($langQuery);
  }


   /**
 * Converts typoscript into a php array, considers includes, doesn't consider constants.
 *
 * @param	string		$ts Typoscript from file or db.
 * @return	array		The typoscript configuration without dots.
 */
  function convertTyposcriptToArray($ts) {
      $tsParser = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Core\\TypoScript\\Parser\\TypoScriptParser');
      $tsParser->parse(t3lib_TSparser::checkIncludeLines($ts));
      return $this->convertTypoScriptArrayToPlainArray($tsParser->setup);
  }


	/**
	 * Removes all trailing dots recursively from TS settings array
	 *
	 * Extbase converts the "classical" TypoScript (with trailing dot) to a format without trailing dot,
	 * to be more future-proof and not to have any conflicts with Fluid object accessor syntax.
	 *
	 * @param	array		$typoScriptArray The TypoScript array (e.g. array('foo' => 'TEXT', 'foo.' => array('bar' => 'baz')))
	 * @return	array
	 * @api
	 * @see typo3/sysext/extbase/Classes/Utility/TypoScript.php or typo3/sysext/extbase/Classes/Service/TypoScriptService.php
	 */
	public function convertTypoScriptArrayToPlainArray(array $typoScriptArray) {
		foreach ($typoScriptArray as $key => &$value) {
			if (substr($key, -1) === '.') {
				$keyWithoutDot = substr($key, 0, -1);
				$hasNodeWithoutDot = array_key_exists($keyWithoutDot, $typoScriptArray);
				$typoScriptNodeValue = $hasNodeWithoutDot ? $typoScriptArray[$keyWithoutDot] : NULL;
				if (is_array($value)) {
					$typoScriptArray[$keyWithoutDot] = $this->convertTypoScriptArrayToPlainArray($value);
					if (!is_null($typoScriptNodeValue)) {
						$typoScriptArray[$keyWithoutDot]['_typoScriptNodeValue'] = $typoScriptNodeValue;
					}
					unset($typoScriptArray[$key]);
				} else {
					$typoScriptArray[$keyWithoutDot] = NULL;
				}
			}
		}
		return $typoScriptArray;
	}


	/**
	 * Returns an array with Typoscript the old way (with dot).
	 *
	 * Extbase converts the "classical" TypoScript (with trailing dot) to a format without trailing dot,
	 * to be more future-proof and not to have any conflicts with Fluid object accessor syntax.
	 * However, if you want to call legacy TypoScript objects, you somehow need the "old" syntax (because this is what TYPO3 is used to).
	 * With this method, you can convert the extbase TypoScript to classical TYPO3 TypoScript which is understood by the rest of TYPO3.
	 *
	 * @param	array		$plainArray An TypoScript Array with Extbase Syntax (without dot but with _typoScriptNodeValue)
	 * @return	array		array with TypoScript as usual (with dot)
	 * @see typo3/sysext/extbase/Classes/Utility/TypoScript.php or typo3/sysext/extbase/Classes/Service/TypoScriptService.php
	 * @api
	 */
	public function convertPlainArrayToTypoScriptArray(array $plainArray) {
		$typoScriptArray = array();
		foreach ($plainArray as $key => $value) {
			if (is_array($value)) {
				if (isset($value['_typoScriptNodeValue'])) {
					$typoScriptArray[$key] = $value['_typoScriptNodeValue'];
					unset($value['_typoScriptNodeValue']);
				}
				$typoScriptArray[$key . '.'] = $this->convertPlainArrayToTypoScriptArray($value);
			} else {
				$typoScriptArray[$key] = $value;
			}
		}
		return $typoScriptArray;
	}


  /**
 * Recursively search a multidimensional array and replace the value at found key.
 *
 * @param	string		$searchKey
 * @param	string		$replaceValue
 * @param	array		$input Multidimensional array
 * @return	array
 */
  function replaceArray($searchKey,$replaceValue,array $input) {
    $return = array();
    foreach ($input as $key => $value) {
        if (is_array($value))
            $value = $this->replaceArray($searchKey,$replaceValue,$value);
        if ($key==$searchKey)
          $return[$key] = $replaceValue;
        else
          $return[$key] = $value;
    }
    return $return;
  }


  /**
 * Get list of all keys of a multidimentional array
 *
 * @param	array		$array Multidimensional array to extract keys from
 * @return	array
 */
  function array_keys_multi(array $array) {
      $keys = array();
      foreach ($array as $key => $value) {
          $keys[] = $key;
          if (is_array($array[$key])) {
              $keys = array_merge($keys, $this->array_keys_multi($array[$key]));
          }
      }
      return $keys;
  }


  /**
 * Converts a multidimensional Array into Typoscript. The result is returned as class-variable.
 *
 * @param	array		$tsArr Multidimensional Typoscript Array
 * @param	integer		$insTab Number of spaces that represents an insert tab
 * @return	void
 */
  function convertArrayToTyposcript($tsArr,$insTab=0) {
    while(list($key,$val)=each($tsArr)) {
      if (is_array($val)) {
        $this->tsResult .= chr(10) . str_repeat(chr(32),($insTab*4));
        $this->tsResult .= substr($key,0,-1).' {';
        $this->convertArrayToTyposcript($val,++$insTab);
        $insTab--;
        $this->tsResult .= chr(10).str_repeat(chr(32),($insTab*4)). '}';

      }
      else {
        $this->tsResult .= chr(10).str_repeat(chr(32),($insTab*4));
        $this->tsResult .= $key.' = '.$val;
      }
    }
  }


  /**
 * Returns autoincrement value of table
 *
 * @param	string		$table DB-table to get next id from
 * @return	integer
 */
  function getNextId($table) {
    $entry = $GLOBALS['TYPO3_DB']->sql_query("SHOW TABLE STATUS WHERE Name = '".$table."'");
    $dbinfo = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($entry);
    return $dbinfo['Auto_increment'];
  }


  /**
 * Reads a file and splits it by a given marker into an array of two parts.
 *
 * @param	string		$path path to template
 * @param	string		$marker name of the marker to split template
 * @return	array		an array with a part before and after the marker
 */
  function readTemplate($path,$marker) {
    $path = t3lib_div::getFileAbsFileName($path);
    if ($path && t3lib_div::validPathStr($path) && @is_file($path)) {
      $file = t3lib_div::getUrl($path);
    }
    return explode($marker, $file);
  }


  /**
 * Cleans file from not replaced markers and writes it.
 *
 * @param	string		$path path to file
 * @param	string		$content of the file
 * @return	void
 */
  function writeFile($path,$content) {
    $content = preg_replace('/###' .$this->marker .'[^#]*###/sim', '', $content);
    t3lib_div::writeFile($path,$content);
  }


  /**
 * Transforms a flexform to an array. Considers all sheets.
 *
 * @param	string		$flexform the flexform-string from db
 * @return	array		the conf-array
 */
  function readFlexform($flexform) {
    if (empty($flexform)) return array();
    $flexformData = t3lib_div::xml2array($flexform);
    $conf = array();
    foreach ( $flexformData['data'] as $sheet => $data ) {
      foreach ( $data as $param => $value ) {
        foreach ( $value as $key => $val ) {
          $conf[$key] = $flexformData['data'][$sheet][$param][$key]['vDEF'];
        }
      }
    }
    return $conf;
  }


  /**
 * Removes directory and its subcontents completely.
 *
 * @param	string		$path path to template-folder
 * @return	boolean		wheather the directory has been removed completely or not
 */
  function reverseRmDir($path) {
      // directory exists
    if ($path && t3lib_div::validPathStr($path) && @t3lib_basicFileFunctions::is_directory($path)) {
        // parse directory, remove files, reverse call for directories, remove directory
      if ($handle = opendir($path)) {
        while (false !== ($file = readdir($handle))) {
          if ($file != "." && $file != "..") {
            if (@is_file($path.'/'.$file)) {
              unlink($path.'/'.$file);
            }
            if (@t3lib_basicFileFunctions::is_directory($path.'/'.$file)) {
              $this->reverseRmDir($path.'/'.$file);
            }
          }
        }
        closedir($handle);
        rmdir($path);
      }
      return true;
    }
    else {
      return false;
    }
  }


  /**
 * Copies a directory and its subcontents iteratively.
 *
 * @param	string		$fromPath path of the source folder
 * @param	string		$toPath path to destination folder
 * @return	boolean		wheather the directory has been copied completely or not
 */
  function reverseCopyDir($fromPath,$toPath) {
      // from directory exists
    if ($fromPath && t3lib_div::validPathStr($fromPath) && @is_dir($fromPath)) {
        // to directory exists, if not try to create
      if (!($toPath && t3lib_div::validPathStr($toPath) && @is_dir($toPath))) {
        $parentToPath = dirname($toPath);
        if (@t3lib_basicFileFunctions::is_directory($parentToPath)) {
          t3lib_div::mkdir($toPath);
        }
        else {
          return false;
        }
      }
        // parse directory, copy files, reverse call for directories
      if ($handle = opendir($fromPath)) {
        while (false !== ($file = readdir($handle))) {
          if ($file != "." && $file != "..") {
            if (@is_file($fromPath.'/'.$file)) {
              copy($fromPath.'/'.$file,$toPath.'/'.$file);
            }
            if (@t3lib_basicFileFunctions::is_directory($fromPath.'/'.$file)) {
              $this->reverseCopyDir($fromPath.'/'.$file,$toPath.'/'.$file);
            }
          }
        }
        closedir($handle);
      }
      return true;
    }
    else {
      return false;
    }
  }


  /**
 * Returns other languages array of table fields.
 *
 * @param	string		$uid the parent record uid
 * @param	string		$table the parent record table
 * @param	string		$columns a comma separated list of columns
 * @return	array		multidimensional array(languages abbreviations->array(fieldname->value))
 */
  function getLangOverlay($uid,$table,$columns) {
    $altContent = array();
    $langQuery = $GLOBALS['TYPO3_DB']->exec_SELECTquery($columns .', sys_language_uid',$table,'l18n_parent='.intval($uid).' AND deleted=0 AND hidden=0','sys_language_uid');

    while (false!==($langRow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($langQuery))) {
      $sysLangId = $langRow['sys_language_uid'];
      $langId = $this->languages[$sysLangId];
      $altContent[$langId] = $langRow;
    }
    $GLOBALS['TYPO3_DB']->sql_free_result($langQuery);
    return $altContent;
  }


  /**
 * Assembles a field for html-template. It makes a type query, checkes whether the selected field-type implies options.
 * If there are options, it checkes whether the options have been updated in the form, then it overwrites db-values with form-values. If options haven't been changed, it uses db-values as source to assemble options.
 * The field markers in the field template are replaced with the values from $fieldRow
 * Adds corresponding typoscriptContent and langContent.
 *
 * @param	array		$fieldRow an array containing the actual field attributes (either form or db)
 * @param	array		$optionDatamap an array containing the form input of option attributes
 * @param	string		$options a comma-separated list with option-id's (form source)
 * @param	integer		$step the formstep
 * @return	string		the field marker substitute
 */
  function assembleField($fieldRow,$options,$step) {
    $typeQuery = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*','tx_formhandlerbackend_type','uid='.intval($fieldRow['is_type']));
    $typeRow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($typeQuery);
    $substituteFieldMarker = str_replace('###fieldname###',$fieldRow['name'],$typeRow['master_template']);
      // does the type imply options?
    if(!empty($typeRow['master_option_template'])) {
      $substituteOptionMarker = '';
      $optionQuery = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*','tx_formhandlerbackend_field_option','field='.intval($fieldRow['uid']).' AND deleted=0 AND hidden=0','','sorting');
      while (false!==($optionRow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($optionQuery))) {
        $substituteOptionMarker .= $this->assembleOption($fieldRow,$typeRow,$optionRow,$step);
      }
      $GLOBALS['TYPO3_DB']->sql_free_result($optionQuery);
    }

    $substituteOptionMarker = str_replace('###fieldname###',$fieldRow['name'],$substituteOptionMarker);
    $substituteFieldMarker = str_replace('###options###',$substituteOptionMarker,$substituteFieldMarker);

      // email content
    $this->mailContent .= str_replace('###fieldname###',$fieldRow['name'],$typeRow['master_email_template']);

      // labels
    $this->langContent['default'] .= '
        			<label index="'.$fieldRow['name'].'">'.$fieldRow['label'].'</label>
        			<label index="'.$fieldRow['name'].'.notice">'.$fieldRow['instructions'].'</label>';
    $altContent = $this->getLangOverlay($fieldRow['uid'],'tx_formhandlerbackend_field','name,label,value,instructions');
    while(list($langId,$val) = each($altContent)) {
      $this->langContent[$langId] .= '
        			<label index="'.$fieldRow['name'].'">'.$altContent[$langId]['label'].'</label>
        			<label index="'.$fieldRow['name'].'.notice">'.$altContent[$langId]['instructions'].'</label>';
    }

      // default value
    if (!empty($fieldRow['value'])) {
      $this->tsContent['defaultValues'][$step]['default'] .= '
        '. $fieldRow['name'] .'.defaultValue = '. $fieldRow['value'];
      reset($altContent);
      while(list($langId,$val) = each($altContent)) {
          $this->tsContent['defaultValues'][$step][$langId] .= '
        '. $fieldRow['name'] .'.defaultValue = '. $altContent[$langId]['value'];
      }
    }

      // validation
    if (!empty($fieldRow['validation'])) {
      $validationQuery = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*','tx_formhandlerbackend_validation','uid IN ('.$fieldRow['validation'].') AND deleted=0 AND hidden=0');
      $errorCheckNum = 1;
      while (false!==($validationRow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($validationQuery))) {
        $this->tsContent['validators'][$step] .= '
          '. $fieldRow['name'] .'.errorCheck.'. $errorCheckNum .' = '. $validationRow['typoscript'];
        if (!empty($validationRow['typoscript_addition'])) {
          $this->tsContent['validators'][$step] .= '
            '. $fieldRow['name'] .'.errorCheck.'. $errorCheckNum .'{
            '. $validationRow['typoscript_addition'] .'
            }';
        }
        $errorLabel = 'error_'.$validationRow['typoscript'];
  
        $this->langContent['default'] .= '
          			<label index="'.$fieldRow['name'].'.validation">'.$validationRow['html5_validation'].'</label>
          			<label index="error_'.$fieldRow['name'].'_'.$validationRow['typoscript'].'">'.$this->langLabels['data']['default'][$errorLabel].'</label>';
  
        reset($this->languages);
        while(list($key,$flag)=each($this->languages)) {
          if ($key==0) continue;
          $this->langContent[$flag] .= '
          			<label index="error_'.$fieldRow['name'].'_'.$validationRow['typoscript'].'">'.$this->langLabels['data'][$flag][$errorLabel].'</label>';
        }
  
        $errorCheckNum++;
      }
    }
    
    return $substituteFieldMarker;
  }

  /**
 * Assembles an option for html-template.
 * The option markers in the option template are replaced with the values from $fieldRow, $typeRow and $optionRow
 * Adds corresponding typoscriptContent and langContent.
 *
 * @param	array		$fieldRow an array containing the actual field attributes (either form or db)
 * @param	array		$typeRow an array containing the actual field type attributes (db)
 * @param	array		$optionRow an array containing the actual option attributes (either form or db)
 * @param	integer		$step the formstep
 * @return	string		the option marker substitute
 */
  function assembleOption($fieldRow,$typeRow,$optionRow,$step) {
    $this->langContent['default'] .= '
        			<label index="'.$fieldRow['name'].'.'.$optionRow['option_value'].'">'.$optionRow['option_label'].'</label>';
    $altContent = $this->getLangOverlay($optionRow['uid'],'tx_formhandlerbackend_field_option','option_value,option_label');
    while(list($langId,$val) = each($altContent)) {
       $this->langContent[$langId] .= '
        			<label index="'.$fieldRow['name'].'.'.$optionRow['option_value'].'">'.$altContent[$langId]['option_label'].'</label>';
    }

    $substituteOptionMarker = str_replace('###option###',$optionRow['option_value'],$typeRow['master_option_template']);
    $substituteOptionMarker = str_replace('###value###',$optionRow['option_value'],$substituteOptionMarker);
    return $substituteOptionMarker;
  }

	/**
	 * Hook called before processing command map
	 * Delete template folder on delete tt_content-element
	 *
	 * @param	string		$cmd: backend command
	 * @param	string		$table: content element table
	 * @param	integer		$id: content element id
	 * @param	string		$value: saved value
	 * @param	object		$reference: content element object
	 * @return	void
	 */
  function processCmdmap_preProcess ($cmd, $table, $id, $value, &$reference) {
      // rm template folder if default element is deleted
    if ($table == 'tt_content' && $cmd == 'delete') {
      $fieldArray = t3lib_BEfunc::getRecord('tt_content',$id,'pi_flexform'," AND list_type='formhandler_backend_pi1' AND sys_language_uid=0");
      if(is_array($fieldArray)) {
        $pluginConf = $this->readFlexform($fieldArray['pi_flexform']);
        if (!empty($pluginConf['pretemplate'])) {
          $templateFolder = PATH_site . $this->templatesFolder . basename($pluginConf['pretemplate']) .'_'. $id;
          $this->reverseRmDir($templateFolder);
        }
      }
    }
    if ($table == 'tt_content' && $cmd == 'localize') {

    }
  }

	/**
	 * Hook called after processing field array.
	 * Save all changes before typo3 for formhandler_backend.
	 * Special treatment for pi_flexform. Prepare template folder.
	 * Exclude form fields and option and newly translated content elements.
	 *
	 * @param	string		$status: backend command status
	 * @param	string		$table: content element table
	 * @param	integer		$id: content element id
	 * @param	string		$fieldArray: saved array
	 * @param	object		$reference: content element object
	 * @return	void
	 */
  function processDatamap_postProcessFieldArray ($status, $table, $id, &$fieldArray, &$reference) {
      // get pid,sys_language_uid
    $pid = (isset($fieldArray['pid']))?$fieldArray['pid']:intval($reference->checkValue_currentRecord['pid']);
    $sys_language_uid = (isset($fieldArray['sys_language_uid']))?$fieldArray['sys_language_uid']:intval($reference->checkValue_currentRecord['sys_language_uid']);

      // don't process new content elements
    if (strpos('NEW',$id)!==false) return false;

      // don't process newly translated content elements
    if ($table == 'tt_content' && $sys_language_uid>0 && $status=='new') {
      return false;
    }

      // remove new-substitutes from field and options
    if (($table == 'tx_formhandlerbackend_field' || $table == 'tx_formhandlerbackend_field_option') && ($sys_language_uid==0 || $status!='new')) {
      $fieldArray = array();
    }

      // check if it's our plugin that updates (including change list_type from and to our plugin)
    if ($table == 'tt_content' && ($fieldArray['list_type'] == 'formhandler_backend_pi1' || $reference->checkValue_currentRecord['list_type'] == 'formhandler_backend_pi1')) {
      $fieldsetDatamap = $reference->datamap['tx_formhandlerbackend_fieldset'];
      $fieldDatamap = $reference->datamap['tx_formhandlerbackend_field'];
      $optionDatamap = $reference->datamap['tx_formhandlerbackend_field_option'];
        // set this tt_content form fieldsets invisible. we need hidden and deleted, because we can only set deleted after save and we need deleted, that fieldsets are not displayed in tt_content-irre
      if ($sys_language_uid==0) $GLOBALS['TYPO3_DB']->exec_UPDATEquery('tx_formhandlerbackend_fieldset','form='.intval($id).' AND sys_language_uid='. intval($sys_language_uid) .' AND pid='.intval($pid),array('hidden'=>1));

        // parse fieldsets
      while (list($fieldsetId,$fieldsetArr)=@each($fieldsetDatamap)) {
        $fields = explode(',',$fieldsetDatamap[$fieldsetId]['fields']);

          // parse fields
        while (list($fieldKey,$fieldId)=@each($fields)) {
          $fieldArr = $fieldDatamap[$fieldId];
          $fieldArr['fieldset'] = $fieldsetId;
          $fieldArr['sorting'] = pow(2,$fieldKey);
          $fieldArr['tstamp'] = time();

            // newid
          if (strpos($fieldId,'NEW')!==FALSE) {
            $fieldArr['uid'] = $this->getNextId('tx_formhandlerbackend_field');
            if ($sys_language_uid==0) $GLOBALS['TYPO3_DB']->exec_INSERTquery('tx_formhandlerbackend_field',$fieldArr);
          }
          else {
            $GLOBALS['TYPO3_DB']->exec_UPDATEquery('tx_formhandlerbackend_field','uid='.intval($fieldId),$fieldArr);
          }
          if (!empty($fieldArr['options'])) {
              // parse options
            $options = explode(',',$fieldArr['options']);
            while (list($optionKey,$optionId)=@each($options)) {
              $optionArr = $optionDatamap[$optionId];
              $optionArr['field'] = $fieldId;
              $optionArr['sorting'] = pow(2,$optionKey);
              $optionArr['tstamp'] = time();
                // newid
              if (strpos($optionId,'NEW')!==FALSE) {
                $optionArr['uid'] = $this->getNextId('tx_formhandlerbackend_field_option');
                $GLOBALS['TYPO3_DB']->exec_INSERTquery('tx_formhandlerbackend_field_option',$optionArr);
              }
              else {
                $GLOBALS['TYPO3_DB']->exec_UPDATEquery('tx_formhandlerbackend_field_option','uid='.intval($optionId),$optionArr);
              }
            }
          }
        }
      }

        // save flexform, include predefined
      $mainId = ($sys_language_uid==0)?$id:$reference->checkValue_currentRecord['l18n_parent'];
      $flexform = (isset($fieldArray['pi_flexform']))?$fieldArray['pi_flexform']:$reference->checkValue_currentRecord['pi_flexform'];
      $pluginConf = $this->readFlexform($flexform);
      $flexformData = t3lib_div::xml2array($flexform);

        // read existing flexform values -> oldPluginConf
      $oldFlexform = $reference->checkValue_currentRecord['pi_flexform'];

      if ($status!='new') {
        $oldPluginConf = $this->readFlexform($oldFlexform);
        if (!empty($oldPluginConf['pretemplate'])) {
            // rm old template folder
          $oldTemplateFolder = PATH_site . $this->templatesFolder . basename($oldPluginConf['pretemplate']) .'_'. $mainId;
          $this->reverseRmDir($oldTemplateFolder);
        }
      }

      if (!empty($pluginConf['pretemplate'])) {
          // copy pretemplates to template-folder
        $pretemplateFolder = t3lib_div::getFileAbsFileName($pluginConf['pretemplate']);
        $templateFolder = PATH_site . $this->templatesFolder . basename($pluginConf['pretemplate']) .'_'. $mainId;
        $templateRelFolder = $this->templatesFolder . basename($pluginConf['pretemplate']) .'_'. $mainId;
        $this->reverseCopyDir($pretemplateFolder,$templateFolder);

          // name of predefinded form
        $tsConfigTemplate = $this->readTemplate($templateFolder .'/'. $this->tsSetupUrl,$this->marker);
        preg_match('/plugin\.Tx_Formhandler\.settings\.predef\.([^ \{]*)[ \{]+.*/', $tsConfigTemplate[0], $predefined);
        $tsConfigBase = $predefined[0];
          // update predefined in flexform-data. we need to add the ts-constants-formname as predefined for formhandler-plugin-integration.
        $flexformData['data']['sDEF']['lDEF']['predefined']['vDEF'] = $predefined[1] .'.';
        $fieldArray['pi_flexform'] = t3lib_div::array2xml($flexformData);
      }
    }
  }

	/**
	 * Hook called after database operations
	 * Update of html-template, lang-template, email-templates, ts-templates, sys_template
	 * Undisplay hidden fieldsets in IRRE
	 *
	 * @param	string		$status: backend command status
	 * @param	string		$table: content element table
	 * @param	integer		$id: content element id
	 * @param	string		$fieldArray: saved array
	 * @param	object		$reference: content element object
	 * @return	void
	 */
  function processDatamap_afterDatabaseOperations (&$status, &$table, &$id, &$fieldArray, &$reference) {
    $sys_language_uid = (isset($fieldArray['sys_language_uid']))?$fieldArray['sys_language_uid']:intval($reference->checkValue_currentRecord['sys_language_uid']);

      // set hidden fieldsets deleted. this is the only way to backup and not display fieldsets, when we change a pretemplate form.
    if ($table=='tx_formhandlerbackend_fieldset') {
      $GLOBALS['TYPO3_DB']->exec_UPDATEquery('tx_formhandlerbackend_fieldset','uid='.intval($id).' AND hidden=1',array('deleted'=>1));
    }

      // don't process newly translated content elements
    if ($table == 'tt_content' && $sys_language_uid>0 && $status=='new') return;

      // check if it's our plugin that updates (including change list_type from and to our plugin)
    if ($table == 'tt_content' && ($fieldArray['list_type'] == 'formhandler_backend_pi1' || $reference->checkValue_currentRecord['list_type'] == 'formhandler_backend_pi1')) {
        // get pid,sys_language_uid
      $pid = (isset($fieldArray['pid']))?$fieldArray['pid']:intval($reference->checkValue_currentRecord['pid']);
      $sys_language_uid = (isset($fieldArray['sys_language_uid']))?$fieldArray['sys_language_uid']:intval($reference->checkValue_currentRecord['sys_language_uid']);
        // foreign languages
      if ($sys_language_uid>0) {
        $id = $reference->checkValue_currentRecord['l18n_parent'];
        $pluginQuery = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*','tt_content','uid='.intval($reference->checkValue_currentRecord['l18n_parent']).' AND tt_content.deleted=0 AND tt_content.hidden=0');
        $pluginRow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($pluginQuery);
        $fieldArray = array('pid'=>$pluginRow['pid'],'sys_language_uid'=>0,'list_type'=>$pluginRow['list_type'],'pi_flexform'=>$pluginRow['pi_flexform']);
        $reference = (object) array('checkValue_currentRecord'=>array('pi_flexform'=>'<xml><data><sDEF><lDEF><null></null></lDEF></sDEF></data></xml>'),'datamap'=>$reference->datamap);
      }

        // read flexform -> plugin_conf
      $flexform = (isset($fieldArray['pi_flexform']))?$fieldArray['pi_flexform']:$reference->checkValue_currentRecord['pi_flexform'];
      $pluginConf = $this->readFlexform($flexform);

      if ($status=='new') {
         // $id (after dboperations is allready saved, but isn't nummerical)
        $id = $this->getNextId('tt_content')-1;
      }

      if (!empty($pluginConf['pretemplate'])) {
        $templateFolder = PATH_site . $this->templatesFolder . basename($pluginConf['pretemplate']) .'_'. $id;
        $templateRelFolder = $this->templatesFolder . basename($pluginConf['pretemplate']) .'_'. $id;

          // read templates
        $htmlTemplate = t3lib_div::getUrl($templateFolder .'/'. $this->htmlTemplateUrl);
        $mailUserTemplate = t3lib_div::getUrl($templateFolder .'/'. $this->mailUserTemplateUrl);
        $mailAdminTemplate = t3lib_div::getUrl($templateFolder .'/'. $this->mailAdminTemplateUrl);
        $langTemplate = t3lib_div::getUrl($templateFolder .'/'. $this->langTemplateUrl);
        $this->langLabels = t3lib_div::xml2array($langTemplate);

          // $tsConfigBase
        $tsConfigTemplate = $this->readTemplate($templateFolder .'/'. $this->tsSetupUrl,$this->marker);
        preg_match('/plugin\.Tx_Formhandler\.settings\.predef\.([^ \{]*)[ \{]+.*/', $tsConfigTemplate[0], $predefined);
        $tsConfigBase = $predefined[0];

          // $tsConstantsBase (third-level)
        $tsConstantsTemplate = $this->readTemplate($templateFolder .'/'. $this->tsConstantsUrl,$this->marker);
        $tsConstantsTemplate = implode(chr(10),$tsConstantsTemplate);
        $this->tsResult = '';
        $tsArr = $this->convertTyposcriptToArray($tsConstantsTemplate);
        $tsKeyArr = $this->array_keys_multi($tsArr);
        $tsConstantsBase = $tsKeyArr[0].'.'.$tsKeyArr[1].'.'.$tsKeyArr[2];

          // define substitutes
        $templateMarkers = array();
        $templateMarkersNames = array();
//        $tsContent = ''; // don't overwrite defaultValues and validators

        // html-template
          // search dynamic markers in html templates
        preg_match_all("/###TEMPLATE_FORM([1-9]+)###.*###TEMPLATE_FORM\\1###/sim", $htmlTemplate, $regs);
        $htmlTemplates = $regs[0];
        $htmlFieldsets = $regs[1];

          // assemble fieldsets
        while (list($key,$fieldsetTemplate)=each($htmlTemplates)) {
            // search for markers
          preg_match_all("/###" .$this->marker. "[_-]?([^#]*)###/sim", $fieldsetTemplate, $regs);
          $templateMarkers = $regs[0];
          $templateMarkersNames = $regs[1];
          foreach ( $templateMarkers as $fieldsetMarker ) {
              // check if corresponding fieldsets exists -> or create it
            $substituteFieldsetMarker = '';
            $fieldsetQuery = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid','tx_formhandlerbackend_fieldset','form='.intval($id).' AND marker LIKE \''.$fieldsetMarker.'\' AND step= \''.$htmlFieldsets[$key].'\' AND pid='.intval($pid));
            if ($GLOBALS['TYPO3_DB']->sql_num_rows($fieldsetQuery)===0 && $sys_language_uid==0) {
                // new fieldset db entry
              $fieldset = array(
                'pid' => intval($pid),
                'tstamp' => time(),
                'sys_language_uid' => $sys_language_uid,
                'form' => intval($id),
                'step' => $htmlFieldsets[$key],
                'marker' => $fieldsetMarker
              );
              $GLOBALS['TYPO3_DB']->exec_INSERTquery('tx_formhandlerbackend_fieldset',$fieldset);
                // tt_content update
              $fieldArray['tx_formhandlerbackend_form']++;
            }
            else {
                // set fieldsets visible
              $fieldsetRow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($fieldsetQuery);
              $GLOBALS['TYPO3_DB']->exec_UPDATEquery('tx_formhandlerbackend_fieldset','uid='.intval($fieldsetRow['uid']),array('hidden'=>0,'deleted'=>0));
              $GLOBALS['TYPO3_DB']->UPDATEquery('tx_formhandlerbackend_fieldset','uid='.intval($fieldsetRow['uid']),array('hidden'=>0,'deleted'=>0));
                // assemble fields
              $substituteFieldsetMarker = '';
                // use db as source
              $fieldQuery = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid, name, label, instructions, value, validation, is_type, options','tx_formhandlerbackend_field','fieldset='. intval($fieldsetRow['uid']) .' AND tx_formhandlerbackend_field.deleted=0 AND tx_formhandlerbackend_field.hidden=0','','sorting');
              while (false!==($fieldRow = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($fieldQuery))) {
                $substituteFieldsetMarker .= $this->assembleField($fieldRow,array(),$htmlFieldsets[$key]);
              }
          		$GLOBALS['TYPO3_DB']->sql_free_result($fieldQuery);
          		  // update template
          		$htmlTemplate = str_replace($fieldsetMarker,$substituteFieldsetMarker,$htmlTemplate);
            }

        		$GLOBALS['TYPO3_DB']->sql_free_result($fieldsetQuery);
          }
        }
          // assemble $htmlTemplate
        $this->writeFile($templateFolder .'/'. $this->htmlTemplateUrl,$htmlTemplate);

        // email templates
          // replace dynamic marker, update templates
        $altFlexform = $this->getLangOverlay($id,'tt_content','pi_flexform');
        reset($this->languages);
        while(list($key,$flag)=each($this->languages)) {
          if ($flag=='default') {
            $mailUserTemplateUrl = $this->mailUserTemplateUrl;
            $tsMailUserTemplateUrl = $mailUserTemplateUrl;
            $mailAdminTemplateUrl = $this->mailAdminTemplateUrl;
            $tsMailAdminTemplateUrl = $mailAdminTemplateUrl;
            $user_email_text = $pluginConf['user_email_text'];
            $this->langContent[$flag] .= '
          			<label index="admin_email_subject">'.$pluginConf['admin_email_subject'].'</label>
          			<label index="user_email_subject">'.$pluginConf['user_email_subject'].'</label>';
          }
          else {
            $mailUserTemplateUrl = str_replace('.', '_'.$flag.'.', $this->mailUserTemplateUrl);
            $mailAdminTemplateUrl = str_replace('.', '_'.$flag.'.', $this->mailAdminTemplateUrl);

            $tsMailConstantsEnd = '
[globalVar=GP:L='.$key.']
  '. $tsConstantsBase .' {
    email.user.user_mail_template = '.$mailUserTemplateUrl.'
    email.admin.admin_mail_template = '.$mailAdminTemplateUrl.'
  }
[global]';
            $altPluginConf = $this->readFlexform($altFlexform[$flag]['pi_flexform']);
            $user_email_text = $altPluginConf['user_email_text'];
            $this->langContent[$flag] .= '
          			<label index="admin_email_subject">'.$altPluginConf['admin_email_subject'].'</label>
          			<label index="user_email_subject">'.$altPluginConf['user_email_subject'].'</label>';
          }
          $mailWrapDynParts = explode('|',$this->sysConf['email_html_dynwrap']);
          $mailWrapParts = explode('|',$this->sysConf['email_html_wrap']);

          $mailUserParts = preg_split('/(###[^#]*###)/',$user_email_text,-1,PREG_SPLIT_DELIM_CAPTURE);
          while (list($key,$mailPart)=each($mailUserParts)) {
              // dynamic
            if ($mailPart=='###'. $this->marker .'###') {
              $mailUserContentPlain .= $mailWrapParts[1].$mailWrapDynParts[0].$this->mailContent.$mailWrapDynParts[1].$mailWrapParts[0];
              $mailUserContentHtml .= $mailWrapParts[1].$mailWrapDynParts[0].$this->mailContent.$mailWrapDynParts[1].$mailWrapParts[0];
            }
              // marker
            elseif (strpos($mailPart,'###')!==false) {
              $mailUserContentPlain .= $mailPart;
              $mailUserContentHtml .= $mailPart;
            }
              // text
            else {
              $label = 'email.user.part.'. $key;
              $start = (strpos($mailPart,chr(32))==0)?' ':'';
              $end = (substr($mailPart,-1)==chr(32))?' ':'';
              $mailUserContentPlain .= $start.'###LLL:'.$label.'.plain###'.$end;
              $mailUserContentHtml .= $start.'###LLL:'.$label.'.html###'.$end;
              $this->langContent[$flag] .= '
          			<label index="'.$label.'.plain">'.$mailPart.'</label>
          			<label index="'.$label.'.html"><![CDATA['.nl2br($mailPart).']]></label>';
            }
          }
          $mailAdminParts = preg_split('/(###[^#]*###)/',$pluginConf['admin_email_text'],-1,PREG_SPLIT_DELIM_CAPTURE);
          while (list($key,$mailPart)=each($mailAdminParts)) {
              // dynamic
            if ($mailPart=='###'. $this->marker .'###') {
              $mailAdminContentPlain .= $mailWrapParts[1].$mailWrapDynParts[0].$this->mailContent.$mailWrapDynParts[1].$mailWrapParts[0];
              $mailAdminContentHtml .= $mailWrapParts[1].$mailWrapDynParts[0].$this->mailContent.$mailWrapDynParts[1].$mailWrapParts[0];
            }
              // marker
            elseif (strpos($mailPart,'###')!==false) {
              $mailAdminContentPlain .= $mailPart;
              $mailAdminContentHtml .= $mailPart;
            }
              // text
            else {
              $label = 'email.admin.part.'. $key;
              $start = (strpos($mailPart,chr(32))==0)?' ':'';
              $end = (substr($mailPart,-1)==chr(32))?' ':'';
              $mailAdminContentPlain .= $start.'###LLL:'.$label.'.plain###'.$end;
              $mailAdminContentHtml .= $start.'###LLL:'.$label.'.html###'.$end;
              $this->langContent[$flag] .= '
          			<label index="'.$label.'.plain">'.$mailPart.'</label>
          			<label index="'.$label.'.html"><![CDATA['.nl2br($mailPart).']]></label>';
            }
          }

          $mailUserTemplate = str_replace('###'. $this->marker .'_html###',$mailWrapParts[0].$mailUserContentHtml.$mailWrapParts[1],$mailUserTemplate);
          $mailUserTemplate = str_replace('###'. $this->marker .'_plain###',str_replace(array('\r\n', '\n', '\r'),'\r\n - ',strip_tags($mailUserContentPlain)),$mailUserTemplate);
          $mailAdminTemplate = str_replace('###'. $this->marker .'_html###',$mailWrapParts[0].$mailAdminContentHtml.$mailWrapParts[1],$mailAdminTemplate);
          $mailAdminTemplate = str_replace('###'. $this->marker .'_plain###',str_replace(array('\r\n', '\n', '\r'),'\r\n - ',strip_tags($mailAdminContentPlain)),$mailAdminTemplate);

          $this->writeFile($templateFolder .'/'. $mailUserTemplateUrl,$mailUserTemplate);
          $this->writeFile($templateFolder .'/'. $mailAdminTemplateUrl,$mailAdminTemplate);

        }

        // lang template
          // assemble $langTemplate
        reset($this->languages);
        while(list($key,$flag)=each($this->languages)) {
          $langTemplate = str_replace('###'. $this->marker .'_'. $flag .'###',$this->langContent[$flag],$langTemplate);
        }
        $this->writeFile($templateFolder .'/'. $this->langTemplateUrl,$langTemplate);

        // ts templates
          // assemble $tsConfigTemplate (defaultValues, validators)
        $tsTemplate = t3lib_div::getUrl($templateFolder .'/'. $this->tsSetupUrl);
        while(list($key,$step)=each($htmlFieldsets)) {
          $tsTemplate = str_replace('###'. $this->marker .'_defaultValues_'. $step .'###',$this->tsContent['defaultValues'][$step]['default'],$tsTemplate);
          $tsTemplate = str_replace('###'. $this->marker .'_validators_'. $step .'###',$this->tsContent['validators'][$step],$tsTemplate);
          reset($this->languages);
          while(list($key,$flag)=each($this->languages)) {
            if ($key==0) continue;
            $tsTemplateEnd .= '
[globalVar=GP:L='. $key .']
  '. $tsConfigBase .'
    preProcessors.1.config.'. $step .' {
    '. $this->tsContent['defaultValues'][$step][$flag] .'
    }
[global]
';
          }
        }

        $this->tsResult = '';
        $tsArr = $this->convertTyposcriptToArray($tsTemplate);
        $tsArr = $this->convertPlainArrayToTypoScriptArray($tsArr);
        $this->convertArrayToTyposcript($tsArr);
        $this->writeFile($templateFolder .'/'. $this->tsSetupUrl,$this->tsResult.$tsTemplateEnd);

          // assemble $tsConstantsTemplate
        $this->tsResult = '';
        $tsArr = $this->convertTyposcriptToArray($tsConstantsTemplate);
            // rootPath
        $tsArr = $this->replaceArray('rootPath',str_replace('EXT:','typo3conf/ext/',$templateRelFolder),$tsArr);
        $tsArr = $this->replaceArray('redirectPage',$pluginConf['redirect_page'],$tsArr);
            // user_email
        $tsArr = $this->replaceArray('user_mail_template',$tsMailUserTemplateUrl,$tsArr);
        $tsArr = $this->replaceArray('user_to_email_field',$pluginConf['user_to_email_field'],$tsArr);
        $tsArr = $this->replaceArray('user_sender_email',$pluginConf['user_sender_email'],$tsArr);
            // admin_email
        $tsArr = $this->replaceArray('admin_mail_template',$tsMailAdminTemplateUrl,$tsArr);
        $tsArr = $this->replaceArray('admin_to_email',$pluginConf['admin_to_email'],$tsArr);
        $tsArr = $this->replaceArray('admin_sender_email',$pluginConf['admin_sender_email'],$tsArr);

        $tsArr = $this->convertPlainArrayToTypoScriptArray($tsArr);
        $this->convertArrayToTyposcript($tsArr);
        $tsConstantsTemplate = $this->tsResult . $tsMailConstantsEnd;
          // write template
        $this->writeFile($templateFolder .'/'. $this->tsConstantsUrl,$tsConstantsTemplate);

        // save page-template (include_static_file gets overwritten by saving manually from backend. include as ts in config and constants?)
        if ($this->sysTemplateWrite) {
          $template = t3lib_BEfunc::getRecordsByField('sys_template','pid',$pid);
          if(is_array($template)) {
              // update by replacing
            $template = $template[0];
            $template['tstamp'] = time();
                // config
            if (preg_match("/(.*?)###fhbe-start###.*?###fhbe-end###(.*)/sim", $template['config'], $temp_parts)) {
              $template['config'] = $temp_parts[1]. "###fhbe-start###
        // automatically generated code, don't edit.
        // this part will be overwritten by saving the formhandler-backend plugin
          <INCLUDE_TYPOSCRIPT: source=\"FILE:". $templateFolder ."/". $this->tsSetupUrl ."\">
      ###fhbe-end###" .$temp_parts[2];
            }
            else {
              $template['config'] .= "
      ###fhbe-start###
        // automatically generated code, don't edit.
        // this part will be overwritten by saving the formhandler-backend plugin
          <INCLUDE_TYPOSCRIPT: source=\"FILE:". $templateFolder ."/". $this->tsSetupUrl ."\">
      ###fhbe-end###";
            }
                // constants
            if (preg_match("/(.*?)###fhbe-start###.*?###fhbe-end###(.*)/sim", $template['constants'], $temp_parts)) {
              $template['constants'] = $temp_parts[1]. "###fhbe-start###
        // automatically generated code, don't edit.
        // this part will be overwritten by saving the formhandler-backend plugin
          <INCLUDE_TYPOSCRIPT: source=\"FILE:". $templateFolder ."/". $this->tsConstantsUrl ."\">
      ###fhbe-end###" .$temp_parts[2];
            }
            else {
              $template['constants'] .= "
      ###fhbe-start###
        // automatically generated code, don't edit.
        // this part will be overwritten by saving the formhandler-backend plugin
          <INCLUDE_TYPOSCRIPT: source=\"FILE:". $templateFolder ."/". $this->tsConstantsUrl ."\">
      ###fhbe-end###";
            }
            $GLOBALS['TYPO3_DB']->exec_UPDATEquery('sys_template','pid='.$pid,$template);
          }
          else {
              // insert new
            $template = array(
              'pid' => $pid,
              'tstamp' => time(),
              'title' => '+ext-formhandler_backend',
                // config
              'config' => "###fhbe-start###
      // automatically generated code, don't edit.
      // this part will be overwritten by saving the formhandler-backend plugin
        <INCLUDE_TYPOSCRIPT: source=\"FILE:". $templateFolder ."/". $this->tsSetupUrl ."\">
    ###fhbe-end###",
                // constants
              'constants' => "###fhbe-start###
      // automatically generated code, don't edit.
      // this part will be overwritten by saving the formhandler-backend plugin
        <INCLUDE_TYPOSCRIPT: source=\"FILE:". $templateFolder ."/". $this->tsConstantsUrl ."\">
    ###fhbe-end###",
            );
            $GLOBALS['TYPO3_DB']->exec_INSERTquery('sys_template',$template);
          }
          if ($sys_language_uid==0) $GLOBALS['TYPO3_DB']->exec_UPDATEquery('tt_content','uid='.intval($id),array('tx_formhandlerbackend_form' => $fieldArray['tx_formhandlerbackend_form']));
        }
      }
    }
  }
}
?>
